@@ -1,5 +1,5 @@
 import sys
-from ..lib.utils import run_test, catch_error_and_exit, run_stackql_query
+from ..lib.utils import run_test, perform_retries, catch_error_and_exit, run_stackql_query, export_vars, show_query
 from ..lib.config import setup_environment, load_manifest, get_global_context_and_providers, get_full_context
 from ..lib.templating import get_queries
 
@@ -15,21 +15,7 @@ class StackQLTestRunner:
         self.manifest = load_manifest(self.stack_dir, self.logger)
         self.stack_name = self.manifest.get('name', stack_dir)
 
-    def _export_vars(self, resource, export, expected_exports, protected_exports):
-        for key in expected_exports:
-            if key not in export:
-                catch_error_and_exit(f"exported key '{key}' not found in exports for {resource['name']}.", self.logger)
-
-        for key, value in export.items():
-            if key in protected_exports:
-                mask = '*' * len(str(value))
-                self.logger.info(f"🔒  set protected variable [{key}] to [{mask}] in exports")
-            else:
-                self.logger.info(f"➡️  set [{key}] to [{value}] in exports")
-
-            self.global_context[key] = value  # Update global context with exported values
-
-    def run(self, dry_run, on_failure):
+    def run(self, dry_run, show_queries, on_failure):
         
         self.logger.info(f"Testing [{self.stack_name}] in [{self.stack_env}] environment {'(dry run)' if dry_run else ''}")
 
@@ -42,41 +28,45 @@ class StackQLTestRunner:
             full_context = get_full_context(self.env, self.global_context, resource, self.logger)    
 
             # get resource queries
-            test_queries, test_query_options = get_queries(self.env, self.stack_dir, 'stackql_queries', resource, full_context, False, self.logger)
+            test_queries, test_query_options = get_queries(self.env, self.stack_dir, 'resources', resource, full_context, False, self.logger)
 
             postdeploy_query = None
             exports_query = None
 
             if 'postdeploy' in test_queries:
                 postdeploy_query = test_queries['postdeploy']
+                postdeploy_retries = test_query_options.get('postdeploy', {}).get('retries', 1)
+                postdeploy_retry_delay = test_query_options.get('postdeploy', {}).get('retry_delay', 0)                
 
             if 'exports' in test_queries:
                 # export variables from resource
                 exports_query = test_queries['exports']
-                exports_retries = test_query_options.get('exports', {}).get('retries', 10)
-                exports_retry_delay = test_query_options.get('exports', {}).get('retry_delay', 10)                
+                exports_retries = test_query_options.get('exports', {}).get('retries', 1)
+                exports_retry_delay = test_query_options.get('exports', {}).get('retry_delay', 0)                
 
             #
             # postdeploy check
             #
-            post_deploy_check_passed = False
+            is_correct_state = False
             if not postdeploy_query:
-                post_deploy_check_passed = True
+                is_correct_state = True
                 if resource.get('type') and resource['type'] == 'query':
                     self.logger.debug(f"❓ test not configured for [{resource['name']}]")
                 else:
                     self.logger.info(f"❓ test not configured for [{resource['name']}]")
             elif dry_run:
-                post_deploy_check_passed = True
+                is_correct_state = True
                 self.logger.info(f"test query for [{resource['name']}]:\n\n{postdeploy_query}\n")
             else:
-                post_deploy_check_passed = run_test(resource, postdeploy_query, self.stackql, self.logger)
+                self.logger.info(f"🔎 checking state for [{resource['name']}]...")
+                show_query(show_queries, postdeploy_query, self.logger)
+                is_correct_state = perform_retries(resource, postdeploy_query, postdeploy_retries, postdeploy_retry_delay, self.stackql, self.logger)
 
             #
             # postdeploy check complete
             #
 
-            if not post_deploy_check_passed:
+            if not is_correct_state:
                 catch_error_and_exit(f"❌ test failed for {resource['name']}.", self.logger)
 
             #
@@ -89,10 +79,14 @@ class StackQLTestRunner:
                     protected_exports = resource.get('protected', [])
 
                     if not dry_run:
-                        self.logger.info(f"exporting variables for [{resource['name']}]...")
+                        self.logger.info(f"📦 exporting variables for [{resource['name']}]...")
+                        show_query(show_queries, exports_query, self.logger)
                         exports = run_stackql_query(exports_query, self.stackql, True, self.logger, exports_retries, exports_retry_delay)
                         self.logger.debug(f"exports: {exports}")
                         
+                        if exports is None:
+                            catch_error_and_exit(f"exports query failed for {resource['name']}", self.logger)                        
+
                         if len(exports) > 1:
                             catch_error_and_exit(f"exports should include one row only, received {str(len(exports))} rows", self.logger)
 
@@ -113,7 +107,7 @@ class StackQLTestRunner:
                                     # Treat as a simple key-value pair
                                     export_data[key] = export.get(key, '')  # Default to empty string if key is missing
 
-                        self._export_vars(resource, export_data, expected_exports, protected_exports)
+                        export_vars(self, resource, export_data, expected_exports, protected_exports)
                     else:
                         self.logger.info(f"dry run exports query for [{resource['name']}]:\n\n{exports_query}\n")
 
